/* coff2flt.c: Convert COFF (or any BFD format) to FLAT binary format
 * Originally obj-res.c
 *
 * (c) 1998, Kenneth Albanowski <kjahds@kjahds.com>
 * (c) 1998, D. Jeff Dionne
 * (c) 1998, The Silver Hammer Group Ltd.
 * (c) 1996, 1997 Dionne & Associates
 * jeff@ryeham.ee.ryerson.ca
 *
 * This is Free Software, under the GNU Public Licence v2 or greater.
 *
 * Relocation added March 1997, Kresten Krab Thorup 
 * krab@california.daimi.aau.dk
 */
 
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <getopt.h>
#include <stdarg.h>

#ifndef PARAMS
#define PARAMS(x) x
#endif

#include <bfd.h>
#include "flat.h"

unsigned long get_long(const void *buf) 
{
     unsigned char *ptr = (unsigned char *) buf;
     
     return (*ptr << 24) | (*(++ptr) << 16) | (*(++ptr) << 8) | *(++ptr);
}

void set_long(void *buf, const unsigned long val) 
{
     unsigned char *ptr = (unsigned char *) buf;
     
     *ptr = (val >> 24) & 0xff;
     *(++ptr) = (val >> 16) & 0xff;
     *(++ptr) = (val >> 8) & 0xff;
     *(++ptr) = val & 0xff;
}
                                   

#ifdef __CYGWIN32__
#define O_PLATFORM O_BINARY
#else
#define O_PLATFORM 0
#endif

#define R_RELLONG       (0x11)  /* Direct 32-bit relocation */

/* libcoff.h would give us access to howtos and bfd_get_reloc_size, but
   the following should be a good enough approximation.  It's exactly right
   for the only reloc type ever likely to be in a .dreloc section.  */
#define RELOC_SIZE 4
      
#define RELOC_ABS_32       0XBe

const char *progname, *filename;
int lineno;

int nerrors = 0;
int nwarnings = 0;

static char where[200];

enum {
  /* Use exactly one of these: */
    E_NOFILE = 0,         /* "progname: " */
      E_FILE = 1,           /* "filename: " */
        E_FILELINE = 2,       /* "filename:lineno: " */
          E_FILEWHERE = 3,      /* "filename:%s: " -- set %s with ewhere() */
          
            /* Add in any of these with |': */
              E_WARNING = 0x10,
                E_PERROR = 0x20
                  };
                  
                  void ewhere (const char *format, ...);
                  void einfo (int type, const char *format, ...);
                  

void
ewhere (const char *format, ...) {
  va_list args;
  va_start (args, format);
  vsprintf (where, format, args);
  va_end (args);
  }

void
einfo (int type, const char *format, ...) {
  va_list args;

  switch (type & 0x0f) {
  case E_NOFILE:
    fprintf (stderr, "%s: ", progname);
    break;
  case E_FILE:
    fprintf (stderr, "%s: ", filename);
    break;
  case E_FILELINE:
    ewhere ("%d", lineno);
    /* fall-through */
  case E_FILEWHERE:
    fprintf (stderr, "%s:%s: ", filename, where);
    break;
    }

  if (type & E_WARNING) {
    fprintf (stderr, "warning: ");
    nwarnings++;
    }
  else
    nerrors++;

  va_start (args, format);
  vfprintf (stderr, format, args);
  va_end (args);

  if (type & E_PERROR)
    perror ("");
  else
    fprintf (stderr, "\n");
  }

                                                                                    

asymbol**
get_symbols (bfd *abfd, long *num)
{
  long storage_needed;
  asymbol **symbol_table;
  long number_of_symbols;
  long i;
  
  storage_needed = bfd_get_symtab_upper_bound (abfd);
	  
  if (storage_needed < 0)
    abort ();
      
  if (storage_needed == 0) {
    return ;
  }

  symbol_table = (asymbol **) malloc (storage_needed);

  number_of_symbols = bfd_canonicalize_symtab (abfd, symbol_table);
  
  if (number_of_symbols < 0) 
    abort ();

  *num = number_of_symbols;
  return symbol_table;
}


long
get_symbol_offset(char *name, asection *sec, asymbol **symbol_table, long number_of_symbols)
{
  long i;
  for (i=0; i<number_of_symbols; i++) {
    if (symbol_table[i]->section == sec) {
      if (!strcmp(symbol_table[i]->name, name)) {
	return symbol_table[i]->value;
      }
    }
  }
  return -1;
}  

#define UNSPECIFIED_RESID 3
struct flat_reloc *
output_relocs (bfd *input_bfd, asymbol **symbols, int number_of_symbols, unsigned long *n_relocs, unsigned char * data)
{
  asection *relocs_sec = bfd_get_section_by_name (input_bfd, ".dreloc");
  bfd_size_type relocs_size;
  long reloc_size;
  arelent **reloc_vector = NULL;
  struct flat_reloc * flat_relocs;
  bfd_byte* relocs;
  long reloc_count, count;
  bfd_byte* rel;
  int i;
  asection* a_sec;
  int nsections = bfd_count_sections(input_bfd);
  unsigned long * resid_from_secndx = malloc(sizeof(long)*nsections);
  int data_start;
  
  *n_relocs = 0;
  
  if (!relocs_sec)
  	return NULL;
  
  for(i=0;i<nsections;i++)
    resid_from_secndx[i] = UNSPECIFIED_RESID;
  
  a_sec = bfd_get_section_by_name (input_bfd, ".bss");
  resid_from_secndx[a_sec->index] = 2;
  a_sec = bfd_get_section_by_name (input_bfd, ".text");
  resid_from_secndx[a_sec->index] = 0;
  a_sec = bfd_get_section_by_name (input_bfd, ".data");
  resid_from_secndx[a_sec->index] = 1;
  
  if (!relocs)
  	return NULL;
  	
  relocs_size = bfd_section_size (input_bfd, relocs_sec);
  
  if (!relocs_size)
  	return NULL;

  relocs = (bfd_byte*) xmalloc (relocs_size);
  if (!bfd_get_section_contents (input_bfd, relocs_sec, relocs, 0, relocs_size)) {
    einfo (E_FILE, "can't read .dreloc' section");
    free (relocs);
    return 0;
    }
    
  reloc_count = 0;
  flat_relocs = 0;
  
  data_start = bfd_section_vma(input_bfd, a_sec);
  printf("Data_start = %d\n", data_start);
  
  for ( rel = relocs; rel < relocs + relocs_size; rel += 8)
  {
    unsigned int type, reloffset;
    int relsecndx, symsecndx;
    asection *sec, *relsec, *symsec;
    CONST char *relsecname, *symsecname;
    char relbuffer[32], symbuffer[32];

    type      = bfd_get_16 (input_bfd, rel);
    relsecndx = bfd_get_16 (input_bfd, rel+2);
    reloffset = bfd_get_16 (input_bfd, rel+4);
    symsecndx = bfd_get_16 (input_bfd, rel+6);

    relsec = symsec = NULL;
    for (sec = input_bfd->sections; sec; sec = sec->next) {
      if (sec->index == relsecndx)  relsec = sec;
      if (sec->index == symsecndx)  symsec = sec;
    }

    sprintf (relbuffer, "[%d?]", (int) relsecndx);
    relsecname = (relsec)? bfd_section_name (input_bfd, relsec) : relbuffer;

    sprintf (symbuffer, "[%d?]", (int) symsecndx);
    symsecname = (symsec)? bfd_section_name (input_bfd, symsec) : symbuffer;

    ewhere ("%s+0x%04x", relsecname, reloffset);

    if (!relsec || resid_from_secndx[relsecndx] != 1) {
      einfo (E_FILEWHERE|E_WARNING,
             "reloc in non-data section %s'", relsecname);
      printf("reloc in non-data setion\n");
      continue;
      }

    reloffset += bfd_section_vma (input_bfd, relsec);

   /* if (reloffset > datares_size - RELOC_SIZE) {
      einfo (E_FILEWHERE|E_WARNING, "reloc location out of range");
      continue;
      }*/

    if (!symsec || resid_from_secndx[symsecndx] == UNSPECIFIED_RESID) {
      einfo (E_FILEWHERE|E_WARNING,
             "reloc relative to strange section %s'", symsecname);
      continue;
      }


/*      einfo (E_FILEWHERE,
             "reloc of type %d in section %s, offset %x, to section %s'", type, relsecname, reloffset, symsecname);
*/

    switch (type) {
    case R_RELLONG: {
/*      unsigned long value = (get_long (datares + reloffset)
                             + bfd_section_vma (input_bfd, symsec));
      unsigned long symresid = resid_from_secndx[symsecndx];
      unsigned int prevoffset = get_short (reloc_heads + 2 * symresid);
      set_short (&datares[reloffset], prevoffset);
      set_short (&datares[reloffset+2], value);
      set_short (reloc_heads + 2 * symresid, reloffset);*/

      reloffset -= data_start;
      
      printf("Offset1 = %x\n", get_long(data+reloffset));
      set_long(data + reloffset, get_long(data+reloffset) - bfd_section_vma(input_bfd, symsec));
      printf("Offset2 = %x\n", get_long(data+reloffset));
      
      flat_relocs = realloc(flat_relocs, (reloc_count+1) * sizeof(struct flat_reloc));
      flat_relocs[reloc_count].value = (resid_from_secndx[symsecndx] & 0x3) | (reloffset << 2);
      fprintf(stderr, "Reloc type = %d, offset = %d, result = %x\n",  resid_from_secndx[symsecndx], reloffset,
      	flat_relocs[reloc_count].value);
      
      reloc_count++;
      
      }
      break;

    default:
      einfo (E_FILEWHERE|E_WARNING, "unknown reloc type 0x%x", type);
      continue;
      }

    }

  free (relocs);

#if 0
  for (count = 0; count < reloc_count; count++)
    {
      arelent *r = &(*reloc_vector)[count];
      asymbol *s = *r->sym_ptr_ptr;
      struct flat_reloc  *p = &flat_relocs[num_prelocs];
      unsigned long offset;
      int type;

#ifdef DEBUG
      printf ("DATA %s: ", r->howto->name);
      printf ("addr=0x%x ", r->address);
      printf ("sym=%s+%x", s->name, s->value);
      printf ("\n");
#endif

      if (!strncmp (r->howto->name, "32", 2))
	{
	  num_prelocs += 1;
	}
      else
	{
	  fprintf (stderr, "warning: skipping relocation");
	  fprintf (stderr, "DATA %s: ", r->howto->name);
	  fprintf (stderr, "addr=0x%x ", r->address);
	  fprintf (stderr, "sym=%s+%x", s->name, s->value);
	  fprintf (stderr, "\n");
	  continue;
	}

      /* offset into data section... */
      offset = r->address - data_section->vma;

      /* s->value value of the symbol.  perhaps we'll need it someday *.

      /* store value of... */
      if (s->section == text_section)
	type = FLAT_RELOC_TYPE_TEXT;

      else if (s->section == data_section)
	type = FLAT_RELOC_TYPE_DATA;

      else if (s->section == bss_section)
	type = FLAT_RELOC_TYPE_BSS;

      else {
        printf("Unknown section type %d\n", s->section);
	abort ();
      }

      p->value = (offset & 0x3fffffff) | (type << 30);
      
      fprintf(stderr, "RELOC: address=%x, type=%d, offset=%x, result=%x\n", r->address, type, offset, p->value);
    }
#endif

    *n_relocs = reloc_count;
    return flat_relocs;
}

#if 0
/* shared lib symbols stuff */

long
get_symbol(char *name, asection *sec, asymbol **symbol_table, long number_of_symbols)
{
  long i;
  for (i=0; i<number_of_symbols; i++) {
    if (symbol_table[i]->section == sec) {
      if (!strcmp(symbol_table[i]->name, name)) {
	return symbol_table[i]->value;
      }
    }
  }
  return -1;
}  

int
output_offset_table(int fd, char *ename, bfd *abfd, asymbol **symbol_table, long number_of_symbols)
{
  long i;
  FILE *ef;
  char buf[80];
  char libname[80];
  long etext_addr;
  long sym_addr;

  int foobar = 0;
  int count = 0;
  signed short *tab = malloc(32768); /* we don't know how many yet*/

  asection *text_section = bfd_get_section_by_name (abfd, ".text");

  if (!(ef = fopen(ename, "r"))) {
    fprintf (stderr,"Can't open %s\n",ename);
    exit(1);
  }

  fgets(libname, 80, ef);

  if (number_of_symbols < 0) {
    fprintf (stderr,"Corrupt symbol table!\n");
    exit(1);
  }

  if ((etext_addr = get_symbol("etext",
			       text_section,
			       symbol_table,
			       number_of_symbols)) == -1) {
    fprintf (stderr,"Can't find the symbol etext\n");
    exit(1);
  }

  fgets(buf, 80, ef);
  while (!feof(ef)) {
    buf[strlen(buf)-1] = 0; /* Arrrgh! linefeeds */

    if ((sym_addr = get_symbol(buf,
			       text_section,
			       symbol_table,
			       number_of_symbols)) == -1) {
      fprintf (stderr,"Can't find the symbol %s\n",buf);
      foobar++;
    } else {
      tab[++count] = htons(sym_addr - etext_addr);
    }
    fgets(buf, 80, ef);
  }

  fclose(ef);

  if (foobar) {
    fprintf (stderr,"*** %d symbols not found\n",foobar);
    exit(10);
  }

  strcpy((char *)&tab[++count],libname);
  tab[0] = htons(count * 2);
  write(fd, tab, count * 2 + strlen(libname) + 2);
  return 0;
}
#endif

static char * program;
static void usage(void)
{  
    fprintf(stderr, "Usage: %s [-v] [-l] [-s stacksize] [-o output.file] bfd.file\n", program);
    exit(2);
}

main(int argc, char *argv[])
{
  int fd;
  bfd *bf;
  asection *s;
  char *ofile=0;
  char *fname = argv[argc - 1];
  int opt;
  int type;
  int i;
  int verbose = 0;
  int stack;

  asymbol **symbol_table;
  long number_of_symbols;

  unsigned long data_len;
  unsigned long bss_len;
  unsigned long text_len;
  unsigned long reloc_len;

  unsigned long data_vma;
  unsigned long bss_vma;
  unsigned long text_vma;

  void *text;
  void *data;
  unsigned long *reloc;
  
  struct flat_hdr hdr;


  program = argv[0];
  progname = argv[0];

  if (argc < 2 || argc > 8 )
  	usage();
  
  stack = 4096;

  while ((opt = getopt(argc, argv, "vs:o:")) != -1) {
    switch (opt) {

    case 'v':
      verbose++;
      break;
    case 'o':
      ofile = optarg;
      break;
    case 's':
      stack = atoi(optarg);
      break;
    default:
      fprintf(stderr, "%s Unknown option\n", argv[0]);
      usage();
      break;
    }
  }
  
  filename = argv[argc-1];

  if (!(bf = bfd_openr(argv[argc -1],0))) {
    fprintf (stderr,"Can't open %s\n",argv[argc -1]);
    exit(1);
  }

  if (bfd_check_format (bf, bfd_object) == 0) {
    printf("File is not an object file\n");
    exit(2);
  }

  symbol_table = get_symbols(bf, &number_of_symbols);

  s = bfd_get_section_by_name (bf, ".text");
  text_vma = s->vma;
  text_len = s->_raw_size;
  text = malloc(text_len);

  if (bfd_get_section_contents(bf,
			       s, 
			       text,
			       0,
			       s->_raw_size) == false) {
    fprintf(stderr, "read error section %s\n", s->name);
    exit(2);
  }

  s = bfd_get_section_by_name (bf, ".data");
  data_vma = s->vma;
  data_len = s->_raw_size;
  data = malloc(data_len);

  if (bfd_get_section_contents(bf,
			       s, 
			       data,
			       0,
			       s->_raw_size) == false) {
    fprintf(stderr, "read error section %s\n", s->name);
    exit(2);
  }

  s = bfd_get_section_by_name (bf, ".bss");
  bss_len = s->_raw_size;
  bss_vma = s->vma;
/*  data_len = s->vma - data_vma; */ /* what is this for? */

  reloc = (unsigned long *)output_relocs (bf, symbol_table, number_of_symbols, &reloc_len, data);

  /* Fill in the binflt_flat header */
  memcpy(hdr.magic,"bFLT",4);
  hdr.version     = htonl(2);
  hdr.entry       = htonl(16 * 4);     /* FIXME */
  hdr.data_start  = htonl(16 * 4 + text_len);
  hdr.data_end    = htonl(16 * 4 + text_len + data_len);
  hdr.bss_end     = htonl(16 * 4 + text_len + data_len + bss_len);
  hdr.stack_size  = htonl(stack); /* FIXME */
  hdr.reloc_start = htonl(16 * 4 + text_len + data_len);
  hdr.reloc_count = htonl(reloc_len);
  bzero(hdr.filler, sizeof(hdr.filler));

  for (i=0; i<reloc_len; i++) reloc[i] = htonl(reloc[i]);

  if (verbose) {
    printf("0x%.4x .text, 0x%.4x .data, 0x%.4x bss\n",text_len,data_len,bss_len);
    printf("0x%.4x relocs\n",reloc_len);
  }
  
  
  if (!ofile) {
    ofile = malloc(strlen(fname) + 5); /* 5 to add suffix */
    strcpy(ofile, fname);
    strcat(ofile, ".bflt");
  }

  if ((fd = open (ofile, O_WRONLY | O_PLATFORM | O_CREAT | O_TRUNC, 0744)) < 0) {
    fprintf (stderr, "Can't open output file %s\n", ofile);
    exit(4);
  }

  write(fd, &hdr, sizeof(hdr));
  write(fd, text, text_len);
  write(fd, data, data_len);
  if (reloc)
    write(fd, reloc, reloc_len*4);
  close(fd);

  exit(0);
}
